/*****************************************************************************/
/* Restore sqlite v3 type 2 (WAL) databases to type 1 (journal)
** placed in the Public Domain 2011-03-27 by the author Richard L Walsh
**
** After using Firefox 4 or Seamonkey 2.1, your Places and Cookies databases
** will be incompatible with previous versions of those apps.  This script
** should restore their compatibility (until you run the new versions again).
**
** - Copy this file to your profile directory
** - Backup cookies.sqlite and places.sqlite
** - Run this script
**
** Note:  if you forget to run this script before starting an earlier version
** of FF or SM, your database files will be renamed *.*.corrupt and new empty
** versions will be created.  To recover, delete the new versions, rename the
** originals, then run this script.
*/
/*****************************************************************************/

db.0 = 2
db.1 = 'cookies.sqlite'
db.2 = 'places.sqlite'

/*****************************************************************************/

Main:
  do ctr = 1 to db.0
    call Restore(db.ctr)
  end
  exit

/*****************************************************************************/

Restore: procedure

  parse arg file

  rc = stream(file,'c','query exists')
  if rc = '' then do
    say 'Error:' file 'not found'
    return
  end

  rc = stream(file,'c','open')
  if rc <> 'READY:' then do
    say 'Error:' file 'could not be opened'
    return
  end

  data = charin(file, 1, 20)
  if left(data, 15) = 'SQLite format 3' then do
    if substr(data, 19, 2) = x2c(0202) then do
      rc = charout(file, x2c(0101), 19)
      if rc <> 0 then do
        say 'Error:' file 'could not be restored'
      end
      else do
        say file 'was successfully restored'
      end
    end
    else do
      if substr(data, 19, 2) = x2c(0101) then do
        say 'Note:' file 'does not need to be restored'
      end
      else do
        say 'Error:' file 'may be corrupt'
      end
    end
  end
  else do
    say 'Error:' file 'does not appear to be a SQLite database'
  end

  rc = stream(file,'c','close')
  if rc <> 'READY:' then do
    say file 'could not be closed'
  end

  return

/*****************************************************************************/

